library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.MATH_REAL.ALL;
use IEEE.NUMERIC_STD.ALL;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity Adder_Add_multiple_I is 

	generic (l : POSITIVE :=5 ; n : integer :=3 ; i : integer :=3);
	
	port ( Address : in std_logic_vector (l-1 downto 0);	
	       Incremento     : in std_logic_vector (i*n-1 downto 0);		
		    Sum     : out std_logic_vector (i*l-1 downto 0));
				
end Adder_Add_multiple_I;


architecture behavioral of Adder_Add_multiple_I is



component Adder_Address_I is 

	generic (l : POSITIVE :=5 ; n : integer :=3);
	
	port ( Address : in std_logic_vector (l-1 downto 0);
	       I_decodificato   : in std_logic_vector (n-1 downto 0);		
		    Sum     : out std_logic_vector (l-1 downto 0));
				
end component Adder_Address_I;



component sottrattore_uno is 

	generic (l : POSITIVE :=5);
	
	port ( Address    : in std_logic_vector (l downto 0);
		    Add_meno_1 : out std_logic_vector (l-1 downto 0));
				
end component;


component Adder_Address_I_lpiu1_bit is 

	generic (l : POSITIVE :=5 ; n : integer :=3);
	
	port ( Address : in std_logic_vector (l-1 downto 0);
	
	       I_decodificato : in std_logic_vector(n-1 downto 0);
		
		    Sum     : out std_logic_vector (l downto 0));
				
end component;


signal sum_prov: std_logic_vector(i*l-1 downto 0);
signal sum_prov_piu_uno: std_logic_vector(l downto 0);


begin 

Gen: for x in 1 to i generate

      
		Gen_1 : if x = 1 generate
		
 					Adder_Address_I_i : Adder_Address_I_lpiu1_bit
			                          generic map (l,n)	 
						                 port map (Address, Incremento(n-1 downto 0), sum_prov_piu_uno);
		
		
		         sottrattore_uno_i : sottrattore_uno
			                          generic map (l)	 
						                 port map (sum_prov_piu_uno, sum_prov(l-1 downto 0));
	   
		end generate Gen_1;
		
		
				Gen_2 : if x > 1 generate
		
 					Adder_Address_I_i : Adder_Address_I
			                          generic map (l,n)	 
						                 port map (sum_prov((x-1)*l -1 downto (x-2)*l), Incremento(x*n-1 downto (x-1)*n), sum_prov(x*l -1 downto (x-1)*l));
		
	   
		end generate Gen_2;
					
		
end generate Gen;


sum <= sum_prov;


end behavioral;