library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.MATH_REAL.ALL;
use IEEE.NUMERIC_STD.ALL;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
USE WORK.MY_PACKAGE.ALL;
USE WORK.MY_PACKAGE_2.ALL;


entity processore_unidirezionale_Singola_move is 

	generic (l	: integer := 3 ; n : POSITIVE :=3 ; X : integer :=4 ; c : integer :=3);	
	
	port ( Start_Address  : in std_logic_vector (l-1 downto 0);
		    vettore_I      : in std_logic_vector (X*integer(ceil(log2(real(n+1))))-1 downto 0);
			 clock          : in std_logic;
			 mux_out_sel    : in std_logic;
			 Enable_we      : in std_logic;
			 Enable_mux_sel : in std_logic;
			 WE_esterno     : in std_logic_vector(2**l downto 0);
			 data_in        : in std_logic_vector(((2**c)*((2**l)+1))-1 downto 0);			 
			 MEM_out        : out std_logic_vector((2**c)-1 downto 0));
			 

end processore_unidirezionale_Singola_move;


architecture behavioral of processore_unidirezionale_Singola_move is


component and_N_1 is 

	generic (k	: integer := 5);	
	
	port ( Enable         : in std_logic;
		    ingresso       : in std_logic_vector (k-1 downto 0);
			 uscita         : out std_logic_vector(k-1 downto 0));
				
end component;


component generazione_segnali_controllo_RAM_unidirezionale_Singola_move is 

	generic (l	: integer := 3 ; n : POSITIVE :=3 ; X : integer :=4);	
	
	port ( Start_Address  : in std_logic_vector (l-1 downto 0);
		    vettore_I      : in std_logic_vector (X*integer(ceil(log2(real(n+1))))-1 downto 0);
			 vettore_WE     : out std_logic_vector(2**l-1 downto 0);
			 vettore_CT_sel : out std_logic_vector(indice_CT_sel((2**l)-1,n)-2 downto 0);
			 vettore_ST_sel : out std_logic_vector(grandezza_ST_sel(l,n)-1 downto 0);
			 muxin_sel_out  : out std_logic_vector(indice_mux_sel(2**l,n)-1 downto 0));
				
end component;



component RAM_up_down_v1 IS

 generic (l : integer := 5; n : integer :=3 ; c : integer :=3);

	PORT( mux_in_sel  :  in std_logic_vector(indice_mux_sel(2**l,n)-1 downto 0);
	      mux_out_sel :  in std_logic;
			WE          :  in std_logic_vector((2**l) downto 0);
			clock       :  in std_logic;
			data_in     :  in std_logic_vector(((2**c)*((2**l)+1))-1 downto 0);
			CT_sel      :  in std_logic_vector(indice_CT_sel((2**l)-1,n)-1 downto 0);
			ST_sel      :  in std_logic_vector(grandezza_ST_sel(l,n)-1 downto 0);
		   MEM_out     :  out std_logic_vector((2**c)-1 downto 0));
			
END component;



component OR_2_generic_input is 

	generic (num_in: integer := 3);	
	
	port ( in_1     : in std_logic_vector(num_in-1 downto 0);
	       in_2     : in std_logic_vector(num_in-1 downto 0);
			 or_out   : out std_logic_vector(num_in-1 downto 0));
				
end component;


component and_N_1_v2 is 

	generic (k	: integer := 5);	
	
	port ( Enable         : in std_logic;
		    ingresso       : in std_logic_vector (k-1 downto 0);
			 uscita         : out std_logic_vector(k downto 0));
				
end component;


signal  vettore_WE_c      : std_logic_vector(2**l-1 downto 0);
signal  vettore_WE_c2     : std_logic_vector(2**l downto 0);
signal  vettore_WE_to_ram : std_logic_vector(2**l downto 0);

signal  vettore_CT_sel_c  : std_logic_vector(indice_CT_sel((2**l)-1,n)-2 downto 0);

signal  vettore_CT_sel_c_2  : std_logic_vector(indice_CT_sel((2**l)-1,n)-1 downto 0);

signal  vettore_ST_sel_c  : std_logic_vector(grandezza_ST_sel(l,n)-1 downto 0);


signal  muxin_sel_out_c   : std_logic_vector(indice_mux_sel(2**l,n)-1 downto 0);
signal  muxin_sel_out_to_ram  : std_logic_vector(indice_mux_sel(2**l,n)-1 downto 0);



begin 


           gen_S         : generazione_segnali_controllo_RAM_unidirezionale_Singola_move
                                      generic map (l,n,x)	
											     port map (  Start_Address,
		                                            vettore_I,
			                                         vettore_WE_c,    
			                                         vettore_CT_sel_c,
			                                         vettore_ST_sel_c, 
			                                         muxin_sel_out_c ); 
							
 vettore_CT_sel_c_2(0)<= '0';
 vettore_CT_sel_c_2(indice_CT_sel((2**l)-1,n)-1 downto 1)<= vettore_CT_sel_c;
 
													
																				
			  And_WE        : and_N_1_v2
                                      generic map (2**l)	
											     port map (Enable_we,
												            vettore_WE_c,
											               vettore_WE_c2);
																
																
			  And_muxsel      : and_N_1
                                      generic map (indice_mux_sel(2**l,n))	
											     port map (Enable_mux_sel,
												            muxin_sel_out_C,
											               muxin_sel_out_to_ram);
																
																			  
			  or_WE      : OR_2_generic_input
                                      generic map (2**l+1)	
											     port map (vettore_WE_c2,
												            WE_esterno,
											               vettore_WE_to_ram);
																
																
		  	  RAM         : RAM_up_down_v1
                                      generic map (l,n,c)	
											     port map (  muxin_sel_out_to_ram,
		                                            mux_out_sel,
			                                         vettore_WE_to_ram,    
			                                         clock,
																  data_in, 
																  vettore_CT_sel_c_2,
			                                         vettore_ST_sel_c, 
			                                         MEM_out); 	
																					 			
										 			
						

end behavioral;