library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.MATH_REAL.ALL;
use IEEE.NUMERIC_STD.ALL;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
USE WORK.MY_PACKAGE.ALL;
USE WORK.MY_PACKAGE_2.ALL;


entity processore_unidirezionale_Singola_move_pipe is 

	generic (l	: integer := 3 ; n : POSITIVE :=3 ; X : integer :=4 ; c : integer :=3);	
	
	port ( Start_Address  : in std_logic_vector (l-1 downto 0);
		    vettore_I      : in std_logic_vector (X*integer(ceil(log2(real(n+1))))-1 downto 0);
			 clock          : in std_logic;
			 mux_out_sel    : in std_logic;
			 Enable_we      : in std_logic;
			 Enable_mux_sel : in std_logic;
			 WE_esterno     : in std_logic_vector(2**l downto 0);
			 data_in        : in std_logic_vector(((2**c)*((2**l)+1))-1 downto 0);			 
			 MEM_out        : out std_logic_vector((2**c)-1 downto 0));
			 

end processore_unidirezionale_Singola_move_pipe;


architecture behavioral of processore_unidirezionale_Singola_move_pipe is


component and_N_1 is 

	generic (k	: integer := 5);	
	
	port ( Enable         : in std_logic;
		    ingresso       : in std_logic_vector (k-1 downto 0);
			 uscita         : out std_logic_vector(k-1 downto 0));
				
end component;


component generazione_segnali_controllo_RAM_unidirezionale_Singola_move is 

	generic (l	: integer := 3 ; n : POSITIVE :=3 ; X : integer :=4);	
	
	port ( Start_Address  : in std_logic_vector (l-1 downto 0);
		    vettore_I      : in std_logic_vector (X*integer(ceil(log2(real(n+1))))-1 downto 0);
			 vettore_WE     : out std_logic_vector(2**l-1 downto 0);
			 vettore_CT_sel : out std_logic_vector(indice_CT_sel((2**l)-1,n)-2 downto 0);
			 vettore_ST_sel : out std_logic_vector(grandezza_ST_sel(l,n)-1 downto 0);
			 muxin_sel_out  : out std_logic_vector(indice_mux_sel(2**l,n)-1 downto 0));
				
end component;



component RAM_up_down_v1 IS

 generic (l : integer := 5; n : integer :=3 ; c : integer :=3);

	PORT( mux_in_sel  :  in std_logic_vector(indice_mux_sel(2**l,n)-1 downto 0);
	      mux_out_sel :  in std_logic;
			WE          :  in std_logic_vector((2**l) downto 0);
			clock       :  in std_logic;
			data_in     :  in std_logic_vector(((2**c)*((2**l)+1))-1 downto 0);
			CT_sel      :  in std_logic_vector(indice_CT_sel((2**l)-1,n)-1 downto 0);
			ST_sel      :  in std_logic_vector(grandezza_ST_sel(l,n)-1 downto 0);
		   MEM_out     :  out std_logic_vector((2**c)-1 downto 0));
			
END component;



component OR_2_generic_input is 

	generic (num_in: integer := 3);	
	
	port ( in_1     : in std_logic_vector(num_in-1 downto 0);
	       in_2     : in std_logic_vector(num_in-1 downto 0);
			 or_out   : out std_logic_vector(num_in-1 downto 0));
				
end component;


component and_N_1_v2 is 

	generic (k	: integer := 5);	
	
	port ( Enable         : in std_logic;
		    ingresso       : in std_logic_vector (k-1 downto 0);
			 uscita         : out std_logic_vector(k downto 0));
				
end component;


component FF IS

GENERIC (j : POSITIVE :=2);

PORT ( reg_in  : in std_logic_vector(j-1 downto 0); 
       clk     : IN STD_LOGIC ;
       reg_out : OUT STD_LOGIC_vector(j-1 downto 0)  ) ;
		 
END component ;


component FF_1 IS

PORT ( reg_in  : in std_logic; 
       clk     : IN STD_LOGIC;
       reg_out : OUT STD_LOGIC) ;
END component ;


signal  vettore_WE_c        : std_logic_vector(2**l-1 downto 0);
signal  vettore_WE_c2       : std_logic_vector(2**l downto 0);
signal  vettore_WE_to_ram   : std_logic_vector(2**l downto 0);
signal  vettore_CT_sel_c    : std_logic_vector(indice_CT_sel((2**l)-1,n)-2 downto 0);
signal  vettore_CT_sel_c_2  : std_logic_vector(indice_CT_sel((2**l)-1,n)-1 downto 0);
signal  vettore_ST_sel_c    : std_logic_vector(grandezza_ST_sel(l,n)-1 downto 0);
signal  muxin_sel_out_c      : std_logic_vector(indice_mux_sel(2**l,n)-1 downto 0);
signal  muxin_sel_out_to_ram : std_logic_vector(indice_mux_sel(2**l,n)-1 downto 0);


signal  start_address_delay : std_logic_vector (l-1 downto 0);
signal  vettore_I_delay     : std_logic_vector (X*integer(ceil(log2(real(n+1))))-1 downto 0);  
signal  mux_out_sel_delay   : std_logic;
signal  Enable_delay        : std_logic;
signal  WE_esterno_delay    : std_logic_vector (2**l downto 0);
signal  data_in_delay       : std_logic_vector (((2**c)*((2**l)+1))-1 downto 0);


signal  mux_out_sel_delay_2   : std_logic;
signal  Enable_delay_2        : std_logic;
signal  WE_esterno_delay_2    : std_logic_vector (2**l downto 0);
signal  data_in_delay_2       : std_logic_vector (((2**c)*((2**l)+1))-1 downto 0);
signal  vettore_WE_c_delay_2        : std_logic_vector(2**l-1 downto 0);
signal  vettore_CT_sel_c_delay_2    : std_logic_vector(indice_CT_sel((2**l)-1,n)-2 downto 0);
signal  vettore_ST_sel_c_delay_2    : std_logic_vector(grandezza_ST_sel(l,n)-1 downto 0);
signal  muxin_sel_out_c_delay_2      : std_logic_vector(indice_mux_sel(2**l,n)-1 downto 0);



signal  vettore_WE_to_ram_delay_3   : std_logic_vector(2**l downto 0);
signal  muxin_sel_out_to_ram_delay_3 : std_logic_vector(indice_mux_sel(2**l,n)-1 downto 0);

signal  vettore_CT_sel_c_delay_3  : std_logic_vector(indice_CT_sel((2**l)-1,n)-1 downto 0);
signal  vettore_ST_sel_c_delay_3    : std_logic_vector(grandezza_ST_sel(l,n)-1 downto 0);


signal  mux_out_sel_delay_3   : std_logic;
signal  data_in_delay_3       : std_logic_vector (((2**c)*((2**l)+1))-1 downto 0);




signal  Enable2_delay        : std_logic;
signal  Enable2_delay_2        : std_logic;

begin 

           FF1 : FF
			        generic map(l)
					  port map ( Start_Address, clock, start_address_delay );
					  
					  
			  FF2 : FF
			        generic map(X*integer(ceil(log2(real(n+1)))))
					  port map ( vettore_I, clock, vettore_I_delay );
					  
			  FF3 : FF_1

					  port map ( mux_out_sel , clock, mux_out_sel_delay );
					  
			  FF4 : FF_1

					  port map ( Enable_we , clock, Enable_delay );
					  			 
								 FF50 : FF_1

					  port map ( Enable_mux_sel , clock, Enable2_delay );
					  
			  FF5 : FF
			        generic map(2**l +1)
					  port map ( WE_esterno , clock, WE_esterno_delay );
					  
			  FF6 : FF
			        generic map(((2**c)*((2**l)+1)))
					  port map ( data_in , clock, data_in_delay );
					  
					  			  



           gen_S         : generazione_segnali_controllo_RAM_unidirezionale_Singola_move
                                      generic map (l,n,x)	
											     port map (  start_address_delay,
		                                            vettore_I_delay,
			                                         vettore_WE_c,    
			                                         vettore_CT_sel_c,
			                                         vettore_ST_sel_c, 
			                                         muxin_sel_out_c ); 
																  
																  
																  
																  
																  
			  FF7 : FF_1
			 
					  port map ( mux_out_sel_delay  , clock, mux_out_sel_delay_2 );
					  
			  FF8 : FF_1
			     
					  port map ( Enable_delay , clock, Enable_delay_2 );
					  
			  FF80 : FF_1
			     
					  port map ( Enable2_delay , clock, Enable2_delay_2 );
					  
			  FF9 : FF
			        generic map(2**l +1)
					  port map ( WE_esterno_delay , clock, WE_esterno_delay_2 );
					  
			  FF10 : FF
			        generic map(((2**c)*((2**l)+1)))
					  port map ( data_in_delay , clock, data_in_delay_2 );
					  
					  
					  
					  
			  FF11 : FF
			        generic map(2**l)
					  port map ( vettore_WE_c  , clock, vettore_WE_c_delay_2  );
					  
			  FF12 : FF
			        generic map(indice_CT_sel((2**l)-1,n)-1)
					  port map ( vettore_CT_sel_c , clock, vettore_CT_sel_c_delay_2 );
					  
			  FF13 : FF
			        generic map(grandezza_ST_sel(l,n))
					  port map ( vettore_ST_sel_c , clock, vettore_ST_sel_c_delay_2 );
					  
			  FF14 : FF
			        generic map(indice_mux_sel(2**l,n))
					  port map ( muxin_sel_out_c, clock, muxin_sel_out_c_delay_2 );
					  
					  
					  
					  
																  
																  
			  
							
 vettore_CT_sel_c_2(0)<= '0';
 vettore_CT_sel_c_2(indice_CT_sel((2**l)-1,n)-1 downto 1)<= vettore_CT_sel_c_delay_2;
 
													

													
													
													
																				
			  And_WE        : and_N_1_v2
                                      generic map (2**l)	
											     port map (Enable_delay_2,
												            vettore_WE_c_delay_2,
											               vettore_WE_c2);
																
																
			  And_muxsel      : and_N_1
                                      generic map (indice_mux_sel(2**l,n))	
											     port map (Enable2_delay_2,
												            muxin_sel_out_c_delay_2,
											               muxin_sel_out_to_ram);
																
																			  
			  or_WE      : OR_2_generic_input
                                      generic map (2**l+1)	
											     port map (vettore_WE_c2,
												            WE_esterno_delay_2,
											               vettore_WE_to_ram);
																
																
					  FF15 : FF_1
		
					  port map ( mux_out_sel_delay_2  , clock, mux_out_sel_delay_3 );
					  
					  
					  FF16 : FF
			        generic map(((2**c)*((2**l)+1)))
					  port map ( data_in_delay_2 , clock, data_in_delay_3 );
					  
					  FF17 : FF
			        generic map(indice_CT_sel((2**l)-1,n))
					  port map ( vettore_CT_sel_c_2 , clock, vettore_CT_sel_c_delay_3 );
					  
			        FF18 : FF
			        generic map(grandezza_ST_sel(l,n))
					  port map ( vettore_ST_sel_c_delay_2 , clock, vettore_ST_sel_c_delay_3 );
					  
					  
					  FF19 : FF
			        generic map(2**l+1)
					  port map ( vettore_WE_to_ram , clock, vettore_WE_to_ram_delay_3 );
					  
			        FF20 : FF
			        generic map(indice_mux_sel(2**l,n))
					  port map ( muxin_sel_out_to_ram, clock, muxin_sel_out_to_ram_delay_3 );
					  														
																
																
																
																
																
																
																
																
																
		  	  RAM         : RAM_up_down_v1
                                      generic map (l,n,c)	
											     port map (  muxin_sel_out_to_ram_delay_3,
		                                            mux_out_sel_delay_3,
			                                         vettore_WE_to_ram_delay_3,    
			                                         clock,
																  data_in_delay_3, 
																  vettore_CT_sel_c_delay_3,
			                                         vettore_ST_sel_c_delay_3, 
			                                         MEM_out); 	
																					 			
										 			
						

end behavioral;